import os
import datetime
from bcrypt import hashpw, gensalt
from flask import Flask, Blueprint
from flask_wtf import CSRFProtect
from flask_bootstrap import Bootstrap
from flask_login import LoginManager
from flask_mail import Mail
from flask_nav import Nav
from projects import base_blueprint, projects_blueprint
from projects.models import User
from projects.helpers import load_user
from projects.nav import nav_bar
from nbi.forms import NBIProjectForm
from nbi.conf import config

app = Flask(__name__)
nbi_blueprint = Blueprint('nbi', __name__,
                          static_folder='static',
                          static_url_path='/nbi/static',
                          template_folder='templates')
app.register_blueprint(base_blueprint)

csrf = CSRFProtect(app)
app.secret_key = os.urandom(24)
Bootstrap(app)

# Login Manager
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'


@login_manager.user_loader
def nbi_load_user(user_id):
    return load_user(user_id)


# Connect mail
mail = Mail(app)

# Setup navbar
nav = Nav()
nav.init_app(app)
nav.register_element('nav_bar', nav_bar)
import nbi.views
app.register_blueprint(nbi_blueprint)
# Turn of the default endpoint
app.register_blueprint(projects_blueprint)

# Onetime authentication reset token salt
app.config['ONETIME_TOKEN_SALT'] = os.urandom(24)

# If debug option
if app.debug:
    # Implement test user
    user = User.get_with_first('email', 'test@nbi.ku.dk')
    if user is None:
        user = User(email='test@nbi.ku.dk',
                    password=hashpw(bytes("test", 'utf-8'),
                                    gensalt()),
                    projects=[], is_active=True,
                    is_authenticated=True, is_anonymous=False,
                    confirmed_on=datetime.datetime.now())
        user.save()



from flask_wtf import FlaskForm
from flask_wtf.file import FileRequired, FileAllowed
from wtforms import StringField, TextAreaField, FileField
from wtforms.validators import DataRequired, Regexp, Optional
from projects.forms import MultiCheckboxField
from nbi.conf import config


class NBIProjectForm(FlaskForm):
    name = StringField('Name', validators=[DataRequired()])
    description = TextAreaField('Description')
    activities = StringField('Activities', validators=[DataRequired()])
    contact = StringField('Contact', validators=[DataRequired()])
    area = MultiCheckboxField('Area', validators=[DataRequired()],
                              choices=[('1', 'Project'),
                                       ('2', 'Bachelor'),
                                       ('3', 'Masters')])
    image = FileField('Project Image', validators=[
        FileRequired(),
        FileAllowed(['jpg', 'png'], 'Only JPG and PNG images are allowed')])
    tags = StringField('Tags', validators=[
        DataRequired(),
        Regexp(r'' + config.get('NBI', 'tags_regex'),
               message=config.get('NBI', 'tags_regex_msg'))])


class NBIProjectSearchForm(FlaskForm):
    name = StringField('Name', validators=[Optional()], default=None)
    activities = StringField('Activities', validators=[Optional()],
                             default=None)
    contact = StringField('Contact', validators=[Optional()], default=None)
    area = StringField('Area', validators=[Optional()], default=None)
    tags = StringField('Tags', validators=[
        Optional(),
        Regexp(r'' + config.get('NBI', 'tags_regex'),
               message=config.get('NBI', 'tags_regex_msg'))], default=None)



from flask import render_template, request, jsonify
from flask_login import login_required, current_user
from nbi import nbi_blueprint, csrf
from projects_base.base.forms import TagsSearchForm
from projects.models import Project
from nbi.forms import NBIProjectForm, NBIProjectSearchForm
from nbi.conf import config


@nbi_blueprint.route('/')
@nbi_blueprint.route('/projects', methods=['GET'])
def projects():
    area_choices = NBIProjectForm.area.kwargs.get('choices')
    form = TagsSearchForm()
    entities = Project.get_all()
    tags = Project.get_top_with('tags', num=5)
    return render_template('nbi/projects.html',
                           title=config.get('PROJECTS', 'title'),
                           grid_header="{}".format(
                               config.get('PROJECTS', 'title')),
                           tags=list(tags.keys()),
                           areas=area_choices, objects=entities, form=form)


@nbi_blueprint.route('/projects', methods=['POST'])
def projects_post():
    form = NBIProjectSearchForm(request.form)
    if form.validate():
        if form.__contains__('csrf_token'):
            form._fields.pop('csrf_token')
        projects = Project.get_all()
        for key, value in form.data.items():
            if value:
                # Reduce dataset by attribute values
                projects = Project.get_with_attr(key, value,
                                                 collection=projects)
        entities = [entity.serialize() for entity in projects]
        return jsonify(data={'projects': entities})

    response = jsonify(data={'danger': ', '.join(
        [msg for attr, errors in form.errors.items() for msg in errors])})
    response.status_code = 400
    return response


@nbi_blueprint.route('/my_projects', methods=['GET'])
@login_required
def my_projects():
    area_choices = NBIProjectForm.area.kwargs.get('choices')
    form = TagsSearchForm()
    entities = [project for project in Project.get_all()
                if project._id in current_user.projects]
    return render_template('nbi/projects.html',
                           title=config.get('PROJECTS', 'title'),
                           grid_header="{}".format(
                               config.get('PROJECTS', 'title')),
                           areas=area_choices, objects=entities, form=form)


@nbi_blueprint.route('/tag/<tag>', methods=['GET'])
def tag_search(tag):
    area_choices = NBIProjectForm.area.kwargs.get('choices')
    form = TagsSearchForm(data={'tag': tag}, csrf_enabled=False)
    entities = {}
    tags = Project.get_top_with('tags', num=5)
    if form.validate():
        entities = Project.get_with_search('tags', form.tag.data)

    # Create new form that has csrf -> enable that tag searches can be done
    # via the returned form
    form = TagsSearchForm(data={'tag': tag})
    return render_template('nbi/projects.html',
                           title=config.get('PROJECTS', 'title'),
                           grid_header="{}".format(
                               config.get('PROJECTS', 'title')),
                           tags=list(tags.keys()),
                           areas=area_choices, objects=entities, form=form)



import os
from setuptools import find_packages
from distutils.core import setup

here = os.path.dirname(__file__)
long_description = open('README.rst').read()

version_ns = {}
with open(os.path.join(here, 'version.py')) as f:
    exec(f.read(), {}, version_ns)

setup(
    name='nbi-projects-site',
    version=version_ns['__version__'],
    long_description=long_description,
    url='https://github.com/rasmunk/nbi-projects-site',
    author='Rasmus Munk',
    author_email='rasmus.munk@nbi.ku.dk',
    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    keywords=['Website', 'Flask', 'MetaData'],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'flask_wtf',
        'WTForms',
        'projects-site'
    ]
)
